import datetime
import os
from zope.catalog.interfaces import ICatalog
from zope.component import queryUtility, getUtility
from zope.interface.verify import verifyObject, verifyClass
from zope.intid.interfaces import IIntIds
from waeup.kofa.interfaces import ICSVExporter
from waeup.kofa.students.export import StudentsExporter
from waeup.kofa.students.student import Student
from waeup.kofa.students.tests.test_batching import StudentImportExportSetup
from waeup.kofa.testing import FunctionalLayer

class StudentsExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(StudentsExporterTest, self).setUp()
        student = Student()
        student.student_id = u'A111111'
        self.app['students'][student.student_id] = self.student = student
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        self.cat = getUtility(ICatalog, name='students_catalog')
        self.intids = getUtility(IIntIds)
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = StudentsExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, StudentsExporter)
        return

    def test_get_as_utility(self):
        # we can get an student exporter as utility
        result = queryUtility(ICSVExporter, name="students")
        self.assertTrue(result is not None)
        return

    def setup_student(self, student):
        # set predictable values for `student`
        student.matric_number = u'M123456'
        student.adm_code = u'my adm code'
        student.clearance_locked = False
        student.clr_code = u'my clr code'
        student.perm_address = u'Studentroad 21\nLagos 123456\n'
        student.reg_number = u'123456'
        student.student_id = u'A111111'
        student.firstname = u'Anna'
        student.lastname = u'Tester'
        student.middlename = u'M.'
        student.date_of_birth = datetime.date(1981, 2, 4)
        student.sex = 'f'
        student.email = 'anna@sample.com'
        student.phone = u'+234-123-12345'
        student.notice = u'Some notice\nin lines.'
        return student

    def test_export(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        exporter = StudentsExporter()
        exporter.export([self.student], self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'adm_code,clearance_locked,clr_code,date_of_birth,email,'
            'firstname,lastname,matric_number,middlename,nationality,'
            'perm_address,phone,reg_number,sex,student_id\r\n'

            'my adm code,0,my clr code,1981-02-04,anna@sample.com,Anna,'
            'Tester,M123456,M.,nigeria,"Studentroad 21\nLagos 123456\n",'
            '+234-123-12345,123456,f,A111111\r\n'
            )
        return
